<template>
  <div ref="chart" class="chart">
    <svg ref="canvas">
    </svg>
  </div>
</template>

<script setup>

import * as d3 from 'd3'

const chart = ref()
const canvas = ref()

const {data: dataRaw, label_x, label_y, formatValue} = defineProps({
  data: {type: String},
  label_x: {type: String},
  label_y: {type: String},
  formatValue: {type: Function, default: (value) => Number(value)}
})

const data = d3.csvParse(dataRaw)

let extraPadding = false

extraPadding = Math.max(...data.map(d=>d.ax.length))

onMounted(()=>{
  (function draw(){
    // Specify the chart’s dimensions.
    const width = 800;
    const height = 600;
    const marginTop = 100;
    const marginRight = 20;
    const marginBottom = 50 + extraPadding*1.2;
    const marginLeft = 50 + extraPadding*4;
    const borderRadius = 1;
  
    const mainColor = "hsl(96,15%,80%)"
    const bgColor = "#344224"
    const highlightColor = "hsl(0,0%,100%)"
    
    const t = d3.transition()
      .duration(1000)
      .ease(d3.easeExpInOut);
  
    // Create the SVG container and call the zoom behavior.
    const svg = d3.select(canvas.value)
      .attr("viewBox", [0, 0, width, height])
      .attr("width", "100%")
      .attr("height", "100%")
      .attr("style", "max-width: 100%; height: auto;")
      // .call(zoom)
      .on("pointerenter pointermove", pointermoved)
      .on("pointerleave", pointerleft)
      .on("touchstart", event => event.preventDefault());
  
    const groups = new Set(data.map(d => d.group));
  
    const color = d3.scaleOrdinal()
      .domain(groups)
      .range(Array.from(groups).map((g,i)=>d3.interpolateRgbBasis(["#D87E28", "#C3C65E", "#487629", "white"])(i/(Array.from(groups).length-1))))
      // .range(Array.from(groups).map((g,i)=>d3.interpolate("#D87E28", "white", "#487629")(i/Array.from(groups).length)))
      .unknown("#ccc");
  
    const swatchSize = 12
  
    const swatches = svg.append("g")
      .attr("class", "legend")
      .selectAll("g")
      .data(groups)
      .enter()
  
    swatches.append("rect")
      .attr("x", marginLeft)
      .attr("y", function(d,i){ return 10 + i*(swatchSize+5)}) // 100 is where the first dot appears. 25 is the distance between dots
      .attr("width", swatchSize)
      .attr("height", swatchSize)
      .style("fill", function(d){ return color(d)})
  
    swatches.append("text")
      .attr("x", marginLeft + swatchSize + 10)
      .attr("y", function(d,i){ return 10 + i*(swatchSize+5) + (swatchSize/2)}) // 100 is where the first dot appears. 25 is the distance between dots
      .style("fill", "white")
      .text(function(d){ return d})
      .attr("text-anchor", "left")
      .style("alignment-baseline", "middle")
  
    const fx = d3.scaleBand()
      .domain(new Set(data.map(d => d.ax)))
      .rangeRound([marginLeft, width - marginRight])
      .paddingInner(0.1);
  
    // Create the horizontal scale and its axis generator.
    // const x = d3.scaleBand()
    //   .domain(d3.sort(data, d => -d.ay).map(d => d.ax))
    //   .range([marginLeft, width - marginRight])
    //   .padding(0.2);
    const x = d3.scaleBand()
        .domain(groups)
        .rangeRound([0, fx.bandwidth()])
        .padding(0.05);
  
    const xAxis = d3.axisBottom(fx).tickSizeOuter(0);
  
    // svg.append("g")
    //   .attr("transform", `translate(0,${height - marginBottom})`)
    //   .call(d3.axisBottom(fx).tickSizeOuter(0))
    //   .call(g => g.selectAll(".domain").remove());
  
    // console.log(d3.max(data, d => d.ay))
  
    // Create the vertical scale.
    const y = d3.scaleLinear()
      .domain([0, 1]).nice()
      .range([height - marginBottom, marginTop]);
  
    const yAxis = d3.axisLeft(y).tickSizeOuter(0).tickFormat(formatValue);
  
    // Append the axes.
    svg.append("g")
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${height - marginBottom})`)
      .call(xAxis)
      .style("color", "lightgray")
      .selectAll("text")
      .style("text-anchor", "end")
      .attr("dx", 0)
      .attr("dy", 10)
      .attr("transform", "rotate(-14)" );
  
    svg.append("g")
      .attr("class", "y-axis")
      .attr("transform", `translate(${marginLeft},0)`)
      .call(yAxis)
      .style("color", "lightgray")
  
    svg.append("text")
      // .attr("transform", "rotate(-90)")
      .attr("y", marginLeft/2)
      .attr("x", 0 - ((height + marginTop - marginBottom) / 2))
      .classed('axislabel', true)
      // .attr("dy", "1em")
      .style("transform", "rotate(-90deg)")
      .style("text-anchor", "middle")
      .style("fill", "white")
      .text(label_y);
  
    const numberOfTicks = 5;
  
    // bars
    svg.append("g")
      .classed("bars", true)
      .selectAll()
      .data(d3.group(data, d => d.ax))
      .join("g")
        .classed("bargroup", true)
        .attr("transform", ([ax]) => `translate(${fx(ax)},0)`)
      .selectAll()
      .data(([, d]) => d)
      .join("rect")
        .classed("bar", true)
        .attr("x", d => x(d.group))
        .attr("y", d => y(d.ay))
        .attr("ax", d => d.ax)
        .attr("ay", d => d.ay)
        .attr("group", d => d.group)
        .attr("width", x.bandwidth())
        .attr("height", d => y(0) - y(d.ay))
        .attr("fill", d => color(d.group));
  
    // // Append the bars.
    // svg.append("g")
    //   .attr("class", "bars")
    //   .attr("fill", mainColor)
    //   .selectAll("rect")
    //   .data(data)
    //   .join("rect")
    //   .classed('bar', true)
    //   .attr("x", d => x(d.ax))
    //   .attr("y", height - marginBottom + borderRadius)
    //   .attr("ax", d => d.ax)
    //   .attr("rx", borderRadius)
    //   .attr("height", 0)
    //   .attr("width", x.bandwidth())
  
    // d3.select(".bars")
    //   .selectAll("rect")
    //   .data(data)
    //   .transition(t)
    //   .attr("height", d => y(0) - y(d.ay) + borderRadius)
    //   .attr("y", d => y(d.ay))
  
    // function zoom(svg) {
    //   const extent = [[marginLeft, marginTop], [width - marginRight, height - marginTop]];
  
    //   svg.call(d3.zoom()
    //     .scaleExtent([1, 8])
    //     .translateExtent(extent)
    //     .extent(extent)
    //     .on("zoom", zoomed));
  
    //   function zoomed(event) {
    //     x.range([marginLeft, width - marginRight].map(d => event.transform.applyX(d)));
    //     svg.selectAll(".bars rect").attr("x", d => x(d.ax)).attr("width", x.bandwidth());
    //     svg.selectAll(".x-axis").call(xAxis);
    //   }
    // }
  
  
  
  
  
  
  
  
  
  
  
  
  
    // Create the tooltip container.
    const tooltip = d3.select(chart.value).append('div')
    tooltip.classed('tooltip', true)
    tooltip.classed('f-copy-stat-text', true)
    tooltip.style("opacity", "0");
  
    // Add the event listeners that show or hide the tooltip.
    const bisect = d3.bisector(d => d.ax).center;
    const tt = () => transition()
      .duration(100)
      .ease(easeExpInOut)
  
    function pointermoved(event) {
      if(event.target.classList.contains('bar')){
        console.log("hit")
        const i = data.find(d=>
        d.ax == event.target.attributes.ax.value &&
        d.group == event.target.attributes.group.value &&
        d.ay == event.target.attributes.ay.value
        )
        const left = x(i.group) + fx(i.ax)
        const top = y(i.ay)
        const bandwidthOffset = x.bandwidth()/2
  
        const tooltipRect = tooltip.node().getBoundingClientRect()
        const tooltipPadding = parseFloat(window.getComputedStyle(tooltip.node(), null).getPropertyValue('padding'))
  
        const pctw = (n) => n / width * 100
        const pcth = (n) => n / height * 100
        const ratio = n => n * svg.node().getBoundingClientRect().width / width
  
        tooltip.style("opacity", "1");
  
        if(pctw(left + bandwidthOffset) + pctw(tooltipRect.width) > 100){
          tooltip.style("right", 0)
          tooltip.style("left", "auto");
        }else{
          tooltip.style("right", "auto")
          tooltip.style("left", `calc(${pctw(left + x.bandwidth()/2)}% - 10px)`);
        }
  
        if(pcth(top) + pcth(tooltipRect.height) + ratio(tooltipPadding) > 100){
          tooltip.style("top", `calc(${pcth(top) - pcth(tooltipRect.height) - tooltipPadding}% - 20px)`);
        }else{
          tooltip.style("top", `calc(${pcth(top)}% + 20px)`);
        }
  
        // const text1 = tooltip.append("strong")
        //   .data([data[i].ax])
        //   .text(d => d);
  
        // const text2 = tooltip.append("div")
        //   .data([data[i].ay])
        //   .text(d => formatValue(d));
  
        tooltip.selectAll("div")
          .data([i.ax])
          .join("div")
          .classed('tooltiptitle', true)
          .text(d => d);
  
        tooltip
          .data([i.group])
          .append("div")
          .text(d => d);
  
        tooltip
          .data([i.ay])
          .append("div")
          .text(d => formatValue(parseFloat(d)));
    
      }else{
        tooltip.style("opacity", "0");
      }
    }
  
    // Reset tooltip location
    function pointerleft() {
      tooltip.style("opacity", "0");
    }
  
    // Hover effects
    // d3.selectAll("rect").on("mouseover", function (e) {
    //   d3.select(e.target).transition(tt).style("fill", highlightColor);
    // }).on("mouseout", function (e) {
    //   d3.select(e.target).transition(tt).style("fill", mainColor);
    // })
  })()

})

</script>

<style lang="scss" scoped>

.chart{
  position: relative;
}

:deep() {
  --chart-font-family: Inter;
  --tooltip-font-family: Inter;
  text{
    font-family: var(--chart-font-family);
  }
  .axislabel, .legend{
    font-size: 12px;
    font-weight: 300;
  }
  .tooltip{
    max-width: 400px;
    .tooltiptitle{
      font-weight: 700;
    }
    pointer-events: none;
    box-sizing: border-box;
    --tooltip-bg-color: #487629;
    --tooltip-color: white;
    position: absolute;
    padding: 12px;
    background-color: var(--tooltip-bg-color);
    color: var(--tooltip-color);
    font-family: var(--tooltip-font-family);
    border-radius: 10px;
    display: flex;
    flex-direction: column;
    gap: 6px;
    transition: 0.5s all ease;
  }
}


</style>